
/************************************************
Java Script Interpreter
Author: DEM0STEN
Contacts and bug reports -> http://demosten.com OR stjordanov@hotmail.com

errors (and warnings) routines

************************************************/

#include <stdio.h>
#include <string.h>
#include "JSI_defs.h"
#include "JSI_errw.h"

typedef struct {
	int iErrNum;        // error number
	DWORD dwLine;       // error line
	DWORD dwColumn;     // error column
} TJSI_Error;

TJSI_Error Error = {JSIE_SUCCESS, JSI_UNKNOWN, JSI_UNKNOWN}; // there can be only ONE error
BOOL bRuntimeError = FALSE;		// indicates runtime error

// DEFINED IN JSI_PARS.C
extern DWORD dwCurLine;		// current line number

/************************************************
JSI_SetError() - set interpretter error
************************************************/
void JSI_SetError(int iErr, DWORD dwColumn)
{
	Error.iErrNum = iErr;
	Error.dwLine = dwCurLine;
	Error.dwColumn = dwColumn;
}
/************************************************
JSI_SetErrorNum() - set interpretter error number
************************************************/
void JSI_EXPORT JSI_SetErrorNum(int iErr)
{
	Error.iErrNum = iErr;
	Error.dwLine = dwCurLine;
	Error.dwColumn = JSI_UNKNOWN;
}
/************************************************
JSI_SetErrorColumn() - set interpretter error column
************************************************/
void JSI_SetErrorColumn(DWORD dwColumn)
{
	Error.dwColumn = dwColumn;
}
/************************************************
JSI_GetError() - return interpretter error code
line and volumn returned if pointers != NULL
************************************************/
int JSI_EXPORT JSI_GetError(DWORD *pdwLine, DWORD *pdwColumn)
{
	if (pdwLine != NULL)
		*pdwLine = Error.dwLine;
	if (pdwColumn != NULL)
		*pdwColumn = Error.dwColumn;
	return Error.iErrNum;
}
/************************************************
JSI_GetErrorStringEnglish() - get error and return string value
lunguage = english
************************************************/
void JSI_EXPORT JSI_GetErrorStringEnglish(char *szString)
{
char szLineCol[50];
	if ((Error.dwLine == JSI_UNKNOWN) && (Error.dwColumn == JSI_UNKNOWN))
		strcpy(szLineCol, "");
	else
		if (Error.dwColumn == JSI_UNKNOWN)
			sprintf(szLineCol, " on line: %u", Error.dwLine);
		else
			if (Error.dwLine == JSI_UNKNOWN)
				sprintf(szLineCol, " on column: %u", Error.dwColumn);
			else
				#ifdef _16bit_CC_
				sprintf(szLineCol, " on line: %u  column: %u", (WORD)Error.dwLine, (WORD)Error.dwColumn);
				#else
				sprintf(szLineCol, " on line: %u  column: %u", Error.dwLine, Error.dwColumn);
				#endif
	switch (Error.iErrNum) {
	case JSIE_SUCCESS: strcpy(szString, "Ok"); return;
	// errors
	case JSIE_FILE_NOT_FOUND:
		strcpy(szString, "File not found"); break;
	case JSIE_INVALID_INPUT_CHAR:
		strcpy(szString, "Invalid input character"); break;
	case JSIE_IO_ERROR:
		strcpy(szString, "I/O error"); break;
	case JSIE_NOT_ENOUGH_MEMORY:
		strcpy(szString, "Not enough memory"); break;
	case JSIE_MISSING_OPEN:
		strcpy(szString, "'(' exprected"); break;
	case JSIE_MISSING_INDEX_ID:
		strcpy(szString, "Missing index identifer '[]'"); break;
	case JSIE_TOO_MANY_OPEN:
		strcpy(szString, "Unexpected '(' found"); break;
	case JSIE_INVALID_IDENTIFER:
		strcpy(szString, "Invalid identifer"); break;
	case JSIE_UNFINISHED_STRING:
		strcpy(szString, "Unfinished string constant"); break;
	case JSIE_INVALID_IF_STATE:
		strcpy(szString, "Invalid if() syntax"); break;
	case JSIE_INVALID_WHILE_STATE:
		strcpy(szString, "Invalid while() syntax"); break;
	case JSIE_INVALID_FOR_STATE:
		strcpy(szString, "Invalid 'for' syntax"); break;
	case JSIE_INVALID_FUNCTION_DEFINITION:
		strcpy(szString, "Invalid function definition"); break;
	case JSIE_OUTSIDE_CONTINUE:
		strcpy(szString, "'continue' used outside of loop statement"); break;
	case JSIE_OUTSIDE_BREAK:
		strcpy(szString, "'break' used outside of loop statement"); break;
	case JSIE_INVALID_OPERATION:
		strcpy(szString, "Invalid operation"); break;
	case JSIE_INVALID_FUNCTION_CALL:
		strcpy(szString, "Invalid function call"); break;
	case JSIE_INVALID_FUNCTION_ARGUMENT:
		strcpy(szString, "Invalid function argument"); break;
	case JSIE_INVALID_VARIABLE_TYPE:
		strcpy(szString, "Invalid variable type"); break;
	case JSIE_INVALID_VARIABLE_NAME:
		strcpy(szString, "Invalid variable name"); break;
	case JSIE_CANNOT_ASSIGN_TO_FUNCTION:
		strcpy(szString, "Cannot assign to function"); break;
	case JSIE_DIVIDE_BY_ZERO:
		strcpy(szString, "Divide by zero"); break;
	case JSIE_EXTR_FUNC_MUST_RETURN:
		strcpy(szString, "External function must return a value"); break;
	case JSIE_EXTR_VAR_IS_READ_ONLY:
		strcpy(szString, "External variable is read only"); break;
	case JSIE_EXTR_VAR_IS_WRITE_ONLY:
		strcpy(szString, "External variable is write only"); break;
	case JSIE_EVAL_WITH_ONE_ARG:
		strcpy(szString, "eval() takes 1 argument"); break;
	case JSIE_PARSEINT_WITH_ONE_ARG:
		strcpy(szString, "parseInt() takes 1 argument"); break;
	case JSIE_PARSEFLOAT_WITH_ONE_ARG:
		strcpy(szString, "parseFloat() takes 1 argument"); break;
	case JSIE_INVALID_STRING_CONST:
		strcpy(szString, "Invalid string constant"); break;
	case JSIE_MISSING_BLOCK_CLOSE:
		strcpy(szString, "Missing block end '}'"); break;
	// defaults
	default: strcpy(szString, "Unknown error"); break;
	} // switch
	strcat(szString, szLineCol);
	if (bRuntimeError)
		strcat(szString, " (runtime error)");
}
