
/************************************************
Java Script Interpreter
Author: DEM0STEN
Contacts and bug reports -> http://demosten.com OR stjordanov@hotmail.com

external interface functions

************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "JSI_defs.h"
#include "JSI_errw.h"
#include "JSI_vars.h"
#include "JSI_trc.h"
#include "JSI_intr.h"
#include "JSI_mem.h"

extern PVar pvGlobalHeap;

TPostfix pfCallResult;

// FROM JSI_VARS.C
/************************************************
GetSubVarResult() - Calculate szName and return it as a postfix
************************************************/
TPostfix GetSubVarResult(char *szName, DWORD *pdwLength, PVar pvLocalHeap);
/************************************************
GetSubVarPos() - get position of first var
examples 'ab.ab'  or 'ab["ab"]'
			^           ^
************************************************/
DWORD GetSubVarPos(char *szName);
/************************************************
GetFirstBreakPosition() -  get position of first '.' or '['
(return 0 for none)
************************************************/
DWORD GetFirstBreakPosition(char *szName);

/************************************************
MakeSubVar() - make sub variable(s) as object(s) (return NULL for error)
************************************************/
PVar MakeSubVar(char *szName, PVar *ppvLocalHeap)
{
BOOL bCallNew;   	// shall we call new FindSubVar
DWORD dwExprLen;	// expression length
TPostfix pfResult;	// expression result
PVar pObj;			// pointer to new obj var

	#ifdef _JSI_DEBUG_
	Trace("\nMakeSubVar(): Searching sub variable: '%s' heap: (%p)\n", szName, *ppvLocalHeap);
	#endif
//	#ifdef _JSI_DEBUG_VAR_
//	PrintVar(obj_vars);
//	#endif
	pfResult = GetSubVarResult(szName, &dwExprLen, *ppvLocalHeap);
	if ((pfResult.Type == opError) || (pfResult.Type == opNone))
		return NULL;
	bCallNew = ((strchr(".[", szName[dwExprLen]) != NULL) &&
				(strlen(&szName[dwExprLen])));
	// make object
	pObj = JSI_SetLocalVar(pfResult.szValue, NULL, 0.0, varObj, NULL, NULL, NULL, ppvLocalHeap);
	JSI_Free(pfResult.szValue);
	if (!pObj)
		return NULL;
	if (bCallNew)
		return MakeSubVar(&szName[dwExprLen], &(pObj->obj_vars));
	else
		return pObj;
}
/************************************************
MakeObjects() - create "object path" to szName
************************************************/
BOOL MakeObjects(char *szName)
{
DWORD dwBreakPtr;
char cTemp, cEndChar;
PVar pObj;
BOOL bResult;

	DWORD dwSubVarPos = GetSubVarPos(szName);
	if (dwSubVarPos == 0)
		return TRUE;
	cEndChar = szName[dwSubVarPos];
	szName[dwSubVarPos] = 0;
	if ((dwBreakPtr = GetFirstBreakPosition(szName)) == 0) {
		szName[dwSubVarPos] = cEndChar;
		return TRUE;
	}
	cTemp = szName[dwBreakPtr];
	szName[dwBreakPtr] = 0;
	pObj = JSI_SetVar(szName, NULL, 0.0, varObj, NULL, NULL, NULL, &pvGlobalHeap);
	szName[dwBreakPtr] = cTemp;
	if (!pObj) {
		szName[dwSubVarPos] = cEndChar;
		return FALSE;
	}
	bResult = (BOOL)MakeSubVar(&szName[dwBreakPtr], &(pObj->obj_vars));
	szName[dwSubVarPos] = cEndChar;
	return bResult;
}
/************************************************
JSI_AddExtrFunction() - add external function
************************************************/
BOOL JSI_EXPORT JSI_AddExtrFunction(char *szName, pJSI_Func pFunction)
{
	#ifdef _JSI_DEBUG_
	Trace("\nAdding function: '%s' -> (%p)\n", szName, pFunction);
	#endif
	MakeObjects(szName);
	if (!JSI_SetVar(szName, NULL, 0.0, varExtFunc, NULL, NULL,
		(PToken)pFunction, &pvGlobalHeap))
		return FALSE;

	return TRUE;
}
/************************************************
JSI_AddExtrVar() - add external variable
************************************************/
BOOL JSI_EXPORT JSI_AddExtrVar(char *szName, pJSI_VarGet pGetFunction, pJSI_VarSet pSetFunction)
{
	#ifdef _JSI_DEBUG_
	Trace("\nAdding variable: '%s' get: (%p) set: (%p)\n",
		szName, pGetFunction, pSetFunction);
	#endif
	if (!JSI_SetVar(szName, (char *)pSetFunction, 0.0, varExtVar, NULL,
		NULL, (PToken)pGetFunction, &pvGlobalHeap))
		return FALSE;

	return TRUE;
}
/************************************************
JSI_GetParam() - get parameter value by number (iVarNum)
************************************************/
TVarType JSI_EXPORT JSI_GetParam(int iVarNum, char **szValue, double *dValue, PVar pVars)
{
PVar pvTemp, pvResult;
char szVarNum[50];

	#ifdef _JSI_DEBUG_
	Trace("\nJSI_GetVar: '%d' -> (%p)\n", iVarNum, pVars);
	#endif
	itoa(iVarNum, szVarNum, 10);
	pvTemp = pvGlobalHeap;
	pvGlobalHeap = pVars;
	pvResult = JSI_FindVar(szVarNum, pVars);
	pvGlobalHeap = pvTemp;
	if (pvResult == NULL)
		return varError;
	switch (pvResult->Type) {
	case varNum: case varStr:
		if (szValue != NULL)
			*szValue = pvResult->szValue;
		break;
	case varFunc: case varObj: case varExtFunc:
		if (szValue != NULL)
			*szValue = pvResult->szName;
		break;
	default: return varError;
	}
	if (dValue != NULL)
		*dValue = pvResult->dValue;
	return pvResult->Type;
}
/************************************************
JSI_SetResult() - set result of external function for use inside JSI
as function result
************************************************/
void JSI_EXPORT JSI_SetResult(char *szValue, double dValue, TVarType Type)
{
	pfCallResult.dValue = dValue;
	switch (Type) {
	case varNum:
		pfCallResult.Type = opConstNum;
		break;
	case varStr: case varFunc: case varObj: case varExtFunc:
		pfCallResult.Type = opConstStr;
		pfCallResult.szValue = CopyString(szValue);
		break;
	default: pfCallResult.Type = opError;
	}
}
