
/************************************************
Java Script Interpreter
Author: DEM0STEN
Contacts and bug reports -> http://demosten.com OR stjordanov@hotmail.com

Preprocess check and some parse routines

************************************************/

#include <stdio.h>
#include <string.h>
#include "JSI_defs.h"
#include "JSI_errw.h"
#include "JSI_mem.h"

const char szJSI_ValidChars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_1234567890!%^&*()-+=[]{}\\|/<>,.'\"; \t\015\012";
const char szJSI_Operators[] = "=+-*^&|!<>/%~()"; // posible first operator chars
// posible identifer chars
const char szJSI_Identifers[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_1234567890.";
const char szJSI_Numbers[] = ".0123456789"; // posible first number chars
const char szJSI_Strings[] = "\"'"; // posible first string const chars

// DEFINED IN JSI_PARS.C
extern DWORD dwCurLine;		// current line number

/************************************************
JSI_PreprocessCheck() - Check for valid characters,
removes comments and 0x0D character, converts \t to ' '
************************************************/
int JSI_PreprocessCheck(char *szMem, char *szScript)
{
int iResult = JSIE_SUCCESS;	// result
BOOL bComment1 = FALSE; // in '//' comment
BOOL bComment2 = FALSE; // in '/**/' comment
DWORD dwSrcIndex = 0, dwDestIndex = 0; // source and destination index
DWORD dwLength = strlen(szMem);
char cTemp;
DWORD dwColumn = 1; // column number

	dwCurLine = 1;
	while (dwSrcIndex < dwLength) {
		cTemp = szMem[dwSrcIndex];
		// check for valid characters
		if (!(bComment1 | bComment2))
			if (strchr(szJSI_ValidChars, cTemp) == NULL) {
				iResult = JSIE_INVALID_INPUT_CHAR;
				JSI_SetError(iResult, dwColumn);
				break;
			}
		dwColumn++;
		dwSrcIndex++;
		switch (cTemp) {
		case '\t': cTemp = ' '; break; // converts \t to ' '
		case 0x0D: dwColumn--; continue;
		case '/' : { // set comment vars
				if (szMem[dwSrcIndex] == '*')
					bComment2 = TRUE;
				if (szMem[dwSrcIndex] == '/')
					bComment1 = TRUE;
			} break;
		case 0x0A: { // unset comment var 2
				if (bComment1)
					bComment1 = FALSE;
				dwCurLine++;
				dwColumn = 1;
			} break;
		case '*' : { // unset comment var 2
				if (!bComment2)
					break;
				if (szMem[dwSrcIndex] == '/')
					bComment2 = FALSE;
				cTemp = ' ';
				szScript[dwDestIndex] = cTemp;
				dwDestIndex++;
				dwColumn++;
				dwSrcIndex++;
			} break;
		} // switch
		if (bComment1)
			continue;
		if ((bComment2) && (cTemp != 0x0A))
			cTemp = ' ';
		szScript[dwDestIndex] = cTemp;
		dwDestIndex++;
	}
	// end
	szScript[dwDestIndex] = 0; // set end sign
	return iResult;
}
/************************************************
JSI_GetVarNameLength() - return length of variable name in the beginning
of szName (return 0 in case of error)
************************************************/
DWORD JSI_GetVarNameLength(char *szName)
{
DWORD dwIndex; // counter
DWORD dwIndexIDCounter = 0; // [] counter
DWORD dwSkobi = 0;			// () counter
BOOL bExit = FALSE;	    	// shall we exit
BOOL bFunction = FALSE;     // is this function

	for (dwIndex = 0; (dwIndex < strlen(szName)) && (!bExit); dwIndex++) {
		switch (szName[dwIndex]) {
		case '(': if (dwIndexIDCounter == 0)  // this is function
					bFunction = TRUE;
			dwSkobi++;
			continue;
		case ')': if ((dwIndexIDCounter != 0) || (dwSkobi == 0)) {
				JSI_SetError(JSIE_MISSING_INDEX_ID, JSI_UNKNOWN);
				return 0;
			}
			dwSkobi--;
			if ((bFunction) && (dwSkobi == 0))
				bExit = TRUE;
			continue;
		case '[': dwIndexIDCounter++; continue;
		case ']': if (dwIndexIDCounter == 0) {
				JSI_SetError(JSIE_MISSING_INDEX_ID, JSI_UNKNOWN);
				return 0;
			}
			dwIndexIDCounter--;
			break;
		case '.': if (strchr(szJSI_Identifers, szName[dwIndex+1]) == NULL) {
				JSI_SetError(JSIE_INVALID_IDENTIFER, JSI_UNKNOWN);
				return 0;
			}
			break;
		case ' ': if ((dwIndexIDCounter == 0) && (dwSkobi == 0)) {
				 dwIndex--;
				 bExit = TRUE;
				 continue;
			}
		default: if ((dwIndexIDCounter == 0) &&
					 (strchr(szJSI_Operators, szName[dwIndex]) != NULL) &&
					 (dwSkobi == 0)) {
					 dwIndex--;
					 bExit = TRUE;
			}
			break;
		} // switch
	} // for
	if (dwIndexIDCounter != 0) {
		JSI_SetError(JSIE_MISSING_INDEX_ID, JSI_UNKNOWN);
		return 0;
	}
	return dwIndex;
}

/************************************************
JSI_GetStringConst() - return length of string constant,
set szString to formated string (return 0 in case of error)
************************************************/
DWORD JSI_GetStringConst(char *szName, char *szString)
{
DWORD dwIndex, dwStrIndex = 0; // counters
BOOL bSingle = FALSE;   // string starts with '
BOOL bDouble = FALSE;   // string starts with "
BOOL bExit = FALSE;	    // shall we exit
char szNum[10], *szLen;
DWORD dwLen;

	switch (szName[0]) { // get start char (must be ' or ")
	case '\'': bSingle = TRUE; break;
	case '"' : bDouble = TRUE; break;
	default  : return 0;
	}
	for (dwIndex = 1; (dwIndex < strlen(szName)) && (!bExit); dwIndex++) {
		switch (szName[dwIndex]) {
		case '\'': 	if (bSingle) {
						bSingle = FALSE;
						bExit = TRUE;
						continue;
					}
					break;
		case '"' :  if (bDouble) {
						bDouble = FALSE;
						bExit = TRUE;
						continue;
					}
					break;
		case '\\':  {
			dwIndex++;
			switch (szName[dwIndex]) {
			case 'r': szString[dwStrIndex] = '\015'; dwStrIndex++; continue;// \r indicates a carriage return.
			case 'n': strcpy(&szString[dwStrIndex], "\015\012"); dwStrIndex += 2; continue; // \n indicates a new line character
			case 'b': szString[dwStrIndex] = '\011'; dwStrIndex++; continue; // \b indicates a backspace.
			case 'f': // \f indicates a a form feed.
			case 't': szString[dwStrIndex] = '\010'; dwStrIndex++; continue; // \t indicates a tab character.
			default:
				strncpy(szNum, &szName[dwIndex-1], 4);
				szNum[4] = 0;
				szNum[0] = '0';
				if (strlen(szNum) == 4) {
					dwLen = strtoul(szNum, &szLen, 0);
					if (*szLen != 0) {
						if (szNum[1] == '0') {
							szString[dwStrIndex] = 0;
							dwStrIndex++;
							dwIndex++;
							continue;
						}
						JSI_SetError(JSIE_INVALID_STRING_CONST, JSI_UNKNOWN);
						return 0;
					}
					szString[dwStrIndex] = (char)dwLen;
					dwStrIndex++;
					dwIndex += 3;
					continue;
				}
				if (szNum[1] == '0') {
					szString[dwStrIndex] = 0;
					dwStrIndex++;
					dwIndex++;
					continue;
				}
				JSI_SetError(JSIE_INVALID_STRING_CONST, JSI_UNKNOWN);
				return 0;
			}
		}
		break;
		} // switch
		szString[dwStrIndex] = szName[dwIndex];
		dwStrIndex++;
	} // for
	szString[dwStrIndex] = 0;
	if (bSingle | bDouble) {
		JSI_SetError(JSIE_UNFINISHED_STRING, JSI_UNKNOWN);
		return 0;
	}
	return dwIndex;
}
/************************************************
JSI_GetFuncName() - return function name (NULL in case of error)
************************************************/
char* JSI_GetFuncName(char *szName)
{
BOOL bString = FALSE; // TRUE = this is a string
BOOL bDouble = FALSE; // TRUE = "this is a string"
char *cResult;		  // end result
int iTempLen;		  // = iLen -1
int iSkobi = 0;       // () counter
int iLen = strlen(szName);

	if (iLen < 2)
		return 0;
	iLen -= 2 ;
	while (iLen > 0) {
		iTempLen = iLen-1;
		switch (szName[iLen]) {
		case '"':
			if (szName[iTempLen] != '\\') {
				if (!bString) {
					bString = TRUE;
					bDouble = TRUE;
				}
				else
					if (bDouble) {
						bString = FALSE;
						bDouble = FALSE;
					}
			}
			else
				iLen--;
			break;

		case '\'':
			if (szName[iTempLen] != '\\') {
				if (!bString) {
					bString = TRUE;
					bDouble = FALSE;
				}
				else
					if (!bDouble) {
						bString = FALSE;
						bDouble = TRUE;
					}
			}
			else
				iLen--;
			break;

		case ')':
			if (!bString)
				iSkobi++;
			break;
		case '(':
			if (!bString)
				iSkobi--;
			break;

		}  // switch
		if (iSkobi < 0)
			break;
		iLen--;
	}
	if (iLen > 0) {
		if ((cResult = JSI_Malloc(iLen+1)) == NULL)
			return NULL;
		memcpy(cResult, szName, iLen);
		cResult[iLen] = 0;
		return cResult;
	}
	return NULL;
}

/*String Literals
A string literal is zero or more characters enclosed in double (") or single (')
quotes. A string must be delimited by quotes of the same type; that is, either
both single quotes or double quotes. The following are examples of string
literals:
  "blah"
  'blah'
  "1234"
  "one line \n another line"
Special Characters
You can use the following special characters in JavaScript string literals:
  \b indicates a backspace.
  \f indicates a a form feed.
  \n indicates a new line character.
  \r indicates a carriage return.
  \t indicates a tab character.
Escaping Characters
You can insert quotes inside of strings by preceding them by a backslash. This
is known as escaping the quotes. For example,
var quote = "He read \"The Cremation of Sam McGee\" by R.W. Service"
document.write(quote)

The result of this would be*/
