
/************************************************
Java Script Interpreter
Author: DEM0STEN
Contacts and bug reports -> http://demosten.com OR stjordanov@hotmail.com

main definitions - for JSI ver.A (beta 3) on Win32

************************************************/

#ifndef _JSI_H_
#define _JSI_H_

#define JSI_WIN32

#ifdef JSI_WIN32
#include <windows.h>
#endif

// MISC definitions
#define JSI_UNKNOWN                 			0xFFFFFFFFUL // unknown line/column

// ERROR definitions and codes
#define JSIE_SUCCESS							0		// "Ok"

// Error definitions - made in groups by type

// general and preprocess errors 1 to 50
#define JSIE_FILE_NOT_FOUND						1		// "File not found"
#define JSIE_INVALID_INPUT_CHAR     			2		// "Invalid input character"
#define JSIE_IO_ERROR							3		// "I/O error"
#define JSIE_NOT_ENOUGH_MEMORY					4		// "Not enough memory"

// parse errors 51 to 100
#define JSIE_MISSING_OPEN						51		// "'(' exprected"
#define JSIE_MISSING_INDEX_ID					52		// "Missing index identifer '[]'"
#define JSIE_TOO_MANY_OPEN						53		// "Unexpected '(' found"
#define JSIE_INVALID_IDENTIFER      			54		// "Invalid identifer"
#define JSIE_UNFINISHED_STRING      			55		// "Unfinished string constant"
#define JSIE_INVALID_STRING_CONST				56		// "Invalid string constant"
#define JSIE_INVALID_IF_STATE					57		// "Invalid if() syntax"
#define JSIE_INVALID_WHILE_STATE				58		// "Invalid while() syntax"
#define JSIE_INVALID_FOR_STATE					59		// "Invalid 'for' syntax"
#define JSIE_INVALID_FUNCTION_DEFINITION		60		// "Invalid function definition"
#define JSIE_OUTSIDE_CONTINUE					61		// "'continue' used outside of loop statement"
#define JSIE_OUTSIDE_BREAK						62		// "'break' used outside of loop statement"
#define JSIE_INVALID_OPERATION					63		// "Invalid operation"
#define JSIE_INVALID_FUNCTION_CALL				64		// "Invalid function call"
#define JSIE_INVALID_FUNCTION_ARGUMENT			65		// "Invalid function argument"
#define JSIE_INVALID_VARIABLE_TYPE				66		// "Invalid variable type"
#define JSIE_INVALID_VARIABLE_NAME				67		// "Invalid variable name"
#define JSIE_CANNOT_ASSIGN_TO_FUNCTION			68		// "Cannot assign to function"
#define JSIE_MISSING_BLOCK_CLOSE				69		// "Missing block end '}'"

// runtime specific errors 101 to 150
#define JSIE_DIVIDE_BY_ZERO						101		// "Divide by zero"

// user defined function errors 151 to 200
#define JSIE_EXTR_FUNC_MUST_RETURN				151		// "External function must return a value"
#define JSIE_EXTR_VAR_IS_READ_ONLY				152		// "External variable is read only"
#define JSIE_EXTR_VAR_IS_WRITE_ONLY				153		// "External variable is write only"
#define JSIE_EVAL_WITH_ONE_ARG					154		// "eval() takes 1 argument"
#define JSIE_PARSEINT_WITH_ONE_ARG				155		// "parseInt() takes 1 argument"
#define JSIE_PARSEFLOAT_WITH_ONE_ARG			156		// "parseFloat() takes 1 argument"

// first number available for user defined errors
#define JSIE_USERBASE							5000

// define operation types
#define	opNone			0		// nothing (no operation)
#define	opError			1		// error
	// constants & vars
#define	opConstNum		4		// number constant
#define	opConstStr		5		// string constant
#define	opVarStr		6		// var as string constant
#define	opVar			7		// variable

typedef int TOpType;			// type definition

// postfix structure - JSI_InterpretMem() & JSI_InterpretFile()
typedef struct TPF {
	char *szValue;					// string of value
	double dValue;                  // double of value
	TOpType Type;                   // type of value
	struct TPF *next;               // pointer to next
} TPostfix;

// define variable types JSI_SetResult()
#define varError		0		// error (use JSI_GetError() to take error number)
#define varNum			1		// number
#define varStr			2		// string
#define varFunc			3		// internal defined function (name)
#define varObj			4		// object (name)
#define varExtFunc		5		// external function (name)
#define varExtVar		6		// external var (name)

typedef int TVarType;			// type definition

// variable structure - not a real one
// you do not have to use this structure directly
// JSI_GetParam() must be used
typedef struct {
	int reserved;
} TVar, *PVar;

#ifdef JSI_WIN32
#define JSI_CALLBACK	CALLBACK
#else
#define JSI_CALLBACK
#endif

typedef int (JSI_CALLBACK *pJSI_Func)(int, PVar);
typedef int (JSI_CALLBACK *pJSI_VarGet)(void);
typedef int (JSI_CALLBACK *pJSI_VarSet)(TVarType, char*, double);

// debug callback function type
typedef int (JSI_CALLBACK *pJSI_DebugCallback)(DWORD);

#ifdef JSI_WIN32
#define JSI_API			WINAPI
#else
#define JSI_API
#endif

// FUNCTIONS
TPostfix JSI_API JSI_InterpretMem(char *szMem);
TPostfix JSI_API JSI_InterpretFile(char *szFileName);
int JSI_API JSI_GetError(DWORD *pdwLine, DWORD *pdwColumn);
void JSI_API JSI_GetErrorStringEnglish(char *szString);
BOOL JSI_API JSI_AddExtrFunction(char *szName, pJSI_Func pFunction);
BOOL JSI_API JSI_AddExtrVar(char *szName, pJSI_VarGet pGetFunction, pJSI_VarSet pSetFunction);
TVarType JSI_API JSI_GetParam(int iVarNum, char **szValue, double *dValue, PVar pVars);
void JSI_API JSI_SetResult(char *szValue, double dValue, TVarType Type);
void JSI_API JSI_SetErrorNum(int iErr);
void JSI_SetDebugCallback(pJSI_DebugCallback func);

#endif
